""" Unit tests for e10_version """
import logging
import os

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import tempfile

from .e10_version import _get_e10_version, NotAnE10Exception

log = logging.getLogger(__name__)


class Test_e10_version(unittest.TestCase):
    """Unit tests for e10_version."""
    # pylint: disable=R0904

    def test_windows(self):
        """On Windows, should raise a NotAnE10Exception."""
        uname = ('Windows', 'some-windows-host', '7', '6.1.7601', 'AMD64', 'Intel64 Family 6 Model 58 Stepping 9, GenuineIntel')
        self.assertRaises(NotAnE10Exception, _get_e10_version, uname, None)

    def test_non_e10_linux(self):
        """On Non-E10 Linux boxen, should raise a NotAnE10Exception."""
        uname = ('Linux', 'some-linux-host', '3.8.0-25-generic', '#37-Ubuntu SMP Thu Jun 6 20:47:07 UTC 2013', 'x86_64')
        self.assertRaises(NotAnE10Exception, _get_e10_version, uname, '/tmp/DOES_NOT_EXIST')

    def test_e10_1_0(self):
        """ 1.0 e10s have no /etc/E10Version """

        uname = ('Linux', 'some-e10', '2.6.33', '#1 Tue Jun 29 15:40:39 UTC 2010', 'armv5tejl', '')

        self.assertEqual('1.0.0', _get_e10_version(uname, '/tmp/DOES_NOT_EXIST'))

    def test_e10_1_1_noversion(self):
        """ HYPOTHETICAL 1.1 e10 without an /etc/E10Version """

        uname = ('Linux', 'some-e10', '3.2.34', '#1 Tue Sep 10 21:11:25 CDT 2013', 'armv5tejl', '')

        self.assertEqual('1.1.0', _get_e10_version(uname, '/tmp/DOES_NOT_EXIST'))

    def test_e10_1_1_withversion(self):
        """ 1.1 e10s have an /etc/E10Version """

        uname = ('Linux', 'some-e10', '3.2.34', '#1 Tue Sep 10 21:11:25 CDT 2013', 'armv5tejl', '')

        tmpname = None
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            tmpname = tmp.name
            tmp.write("1.1.0")

        self.assertEqual('1.1.0', _get_e10_version(uname, tmpname))

        try:
            os.remove(tmpname)
        except OSError:
            log.exception("Error removing tmpfile %s, you'll have to clean up yourself...", tmpname)

    def test_e10_2_0_withversion(self):
        """ HYPOTHETICAL: future e10s have an /etc/E10Version, this trumps other detection. """

        uname = ('Linux', 'some-e10', '3.8.34', '#1 Tue Sep 10 21:11:25 CDT 2013', 'armv5tejl', '')

        tmpname = None
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            tmpname = tmp.name
            tmp.write("2.0.0")

        self.assertEqual('2.0.0', _get_e10_version(uname, tmpname))

        try:
            os.remove(tmpname)
        except OSError:
            log.exception("Error removing tmpfile %s, you'll have to clean up yourself...", tmpname)

    def test_e10_2_0_with_no_version(self):
        """
        HYPOTHETICAL: If this code is run on a future e10 that does
        **not** have an /etc/E10Version, we can't determine that it's an E10.
        """

        uname = ('Linux', 'some-e10', '3.8.34', '#1 Tue Sep 10 21:11:25 CDT 2013', 'armv5tejl', '')

        self.assertRaises(NotAnE10Exception, _get_e10_version, uname, '/tmp/DOES_NOT_EXIST')


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG, format='%(asctime)-15s %(levelname)-8s %(name)-8s %(message)s')
    unittest.main()
