""" This module determines 'What version of E10 is this that I see before me?' """

import logging
import platform


log = logging.getLogger(__name__)


class NotAnE10Exception(Exception):
    """Exception to be raised when we detect that we're not running on an E10."""
    pass


def get_e10_version():  # pragma: no cover
    """
    Return the E10 version of the current platform.

    Returns:
        String: version number in #.#.# format if the current platform is an E10.

    Raises:
        NotAnE10Exception: If the platform is not an E10


    Checks:
        0) Is our OS Linux?
        1) Does an /etc/E10Version file exist?
        2) Are we on an 'armv5tejl' machine?
        3) Do we recognize the kernel version? (2.6.33 for v1.0, 3.2.34 for v1.1 (alpha))

    Note:
        A sufficiently modified E10 (custom kernel, say) may not be recognized.
    """

    return _get_e10_version(platform.uname(), '/etc/E10Version')


def _get_e10_version(uname, e10version_path):
    """ Takes arguments for unit testing purposes. """

    log.debug("Determining E10 Version for: %s, %s, %s, %s", uname[0], uname[4], uname[2], e10version_path)

    # The zeroth test: Are we on Linux? (sysname == 'Linux')
    if uname[0] != 'Linux':
        log.error("Can't be an E10, not running on Linux!")
        raise NotAnE10Exception("Can't be an E10, not running on Linux!")

    # The first test: Is there an /etc/E10Version file?
    # If there is, its value trumps any detection we'd do.
    try:
        with open(e10version_path, 'r') as f:
            e10version = f.readline().strip()
            log.info("E10 Version: %s", e10version)
            return e10version
    except IOError:
        # This isn't fatal, it might be a v1.0 E10
        log.info("Can't be a newer E10, there's no /etc/E10Version file.")

    # The second test: Are we on the right machine type?
    # (machine = 'armv5tejl')
    if uname[4] != 'armv5tejl':
        log.error("Can't be an E10, not the right machine type!")
        raise NotAnE10Exception("Can't be an E10, not the right machine type!")

    # The third test: What's our kernel version?
    if uname[2] == '2.6.33':
        # Probably a 1.0 E10
        e10version = '1.0.0'
        log.info("E10 Version: %s", e10version)
        return e10version
    elif uname[2] == '3.2.34':
        # Probably a 1.1 E10, but why couldn't we find /etc/E10Version?
        e10version = '1.1.0'
        log.info("E10 Version: %s", e10version)
        return e10version

    log.error("Could not reliably determine whether this is an E10.")
    raise NotAnE10Exception("Could not reliably determine whether this is an E10.")
